Srovnani <- function(n) {
  ## funkce pro srovnani hustoty a pp fce pro normalni a Studentovo rozdeleni
  ## n ... pocet stupnu volnosti Studentova rozdeleni
  mu <- 0
  sigma <- 1
  xfit <- seq(-3, 3, length = 512)
  yfit <- dnorm(xfit, mu, sigma)
  zfit <- dt(xfit, n)

  par(mfrow = c(1, 2), mar = c(5, 4, 2, 1))
  plot(xfit, yfit,
    type = "l", col = "blue", xlab = "",
    ylab = "f(x)", main = "", las = 1, ylim = c(0, 0.5)
  )
  lines(xfit, zfit, col = "red")
  mtext("x", side = 1, line = 2.5)
  mtext(paste("n =", n), side = 1, line = 3.7)
  legend("topright",
    col = c("blue", "red"), bty = "n", lty = 1,
    legend = c("Normalni rozdeleni", "Studentovo rozdeleni")
  )

  yfit <- pnorm(xfit, mu, sigma)
  zfit <- pt(xfit, n)
  plot(xfit, yfit,
    type = "l", col = "blue", xlab = "", ylab = "F(x)",
    main = "", las = 1
  )
  lines(xfit, zfit, col = "red")
  mtext("x", side = 1, line = 2.5)
  mtext(paste("n =", n), side = 1, line = 3.7)
  legend("bottomright",
    col = c("blue", "red"), bty = "n", lty = 1,
    legend = c("Normalni rozdeleni", "Studentovo rozdeleni")
  )
}

VyberovyRozptyl <- function(X) {
  ## funkce pocita vyberovy rozptyl vektoru X s pouzitim n
  n <- length(X)
  1 / n * (sum((X - mean(X))^2))
}

RozdeleniSn2Sn <- function(sigma, n, M = 1000) {
  ## porovnani exaktniho a asymptotickeho rozdeleni vyberoveho rozptylu a vyberove sm. odchylky
  ## sigma ... sm. odchylka normalniho rozdeleni
  ## n ... rozsah
  ## M ... pocet generovanych nahodnych vyberu

  Sn2 <- replicate(M, VyberovyRozptyl(rnorm(n, 0, sigma)))
  Sn <- sqrt(Sn2)
  xfit_Sn2 <- seq(from = min(Sn2) - 1, to = max(Sn2) + 1, length = 512)
  xfit_Sn <- seq(min(Sn) - 1, max(Sn) + 1, length = 512)

  yfit_e <- dgamma(xfit_Sn2, shape = n / 2, scale = 2 * sigma^2 / n)
  yfit_a <- dnorm(xfit_Sn2, sigma^2, sqrt(2 * sigma^4 / n))
  zfit_e <- VGAM::dgengamma.stacy(xfit_Sn, scale = sqrt(2 * sigma^2 / n), d = 2, k = n / 2)
  zfit_a <- dnorm(xfit_Sn, sigma, sqrt(sigma^2 / (2 * n)))

  par(mfrow = c(1, 2), mar = c(5, 4, 2, 1))
  hist(Sn2,
    prob = T, main = "", ylab = "hustota", xlab = "",
    col = "khaki1", ylim = c(0, as.numeric(max(yfit_e, yfit_a))),
    density = 25, border = "khaki4", las = 1
  )
  box(bty = "o")
  lines(xfit_Sn2, yfit_e, col = "red") # exaktni
  lines(xfit_Sn2, yfit_a, col = "blue") # asymptoticke
  mtext(expression(S[n]^2), side = 1, line = 2.5, cex = 0.8)
  mtext(bquote(paste(mu == 0, ", ", sigma^2 == .(sigma^2), ", ", n == .(n))),
    side = 1, line = 3.7, cex = 0.8
  )
  legend("topright",
    col = c("red", "blue"),
    legend = c("exakt.", "asympt."), lwd = c(2, 2), bty = "n"
  )

  hist(Sn,
    prob = T, main = "", ylab = "hustota",
    xlab = "", col = "khaki1", ylim = c(0, as.numeric(max(zfit_e, zfit_a)) + 0.1),
    density = 25, border = "khaki4", las = 1
  )
  box(bty = "o")
  lines(xfit_Sn, zfit_e, col = "red") # exaktni
  lines(xfit_Sn, zfit_a, col = "blue") # asymptoticke
  mtext(expression(S[n]), side = 1, line = 2.5, cex = 0.8)
  mtext(bquote(paste(mu == 0, ", ", sigma^2 == .(sigma^2), ", ", n == .(n))),
    side = 1, line = 3.7, cex = 0.8
  )
  legend("topright",
    col = c("red", "blue"),
    legend = c("exakt.", "asympt."), lwd = c(2, 2), bty = "n"
  )
}

DisMu <- function(n, mu = 5, mu2 = mu, sigma = sqrt(15),
                  sigma2 = sigma, M = 100, p = 0.9, alpha = 0.05) {
  ## funkce pro vypocet Waldova DIS pro stredni hodnotu pri znamem rozptylu
  ## n ... rozsah nah. vyberu
  ## mu, mu2 ... stredni hodnoty rozdeleni
  ## sigma, sigma2 ... sm. odchylky rozdeleni
  ## M ... pocet nahodnych vyberu
  ## p ... parametr smesi rozdeleni
  ## alpha ... hladina vyznamnosti

  X <- matrix(NA, M, n)
  for (i in 1:M) {
    bin <- rbinom(n, 1, p)
    X[i, ][bin == 1] <- rnorm(sum(bin == 1), mu, sigma)
    X[i, ][bin == 0] <- rnorm(sum(bin == 0), mu2, sigma2)
  }
  m <- apply(X, 1, mean)
  #s <- apply(X, 1, sd)  
  dh <- m - qnorm(1 - alpha / 2) * sigma / sqrt(n)
  hh <- m - qnorm(alpha / 2) * sigma / sqrt(n)

  rozhodnuti <- (dh < mu) & (mu < hh)
  a_pst_pokryti <- sum(rozhodnuti) / M
  a_alpha <- 1 - a_pst_pokryti

  x <- 1:M
  par(mar = c(5, 4, 1, 1))
  plot(x, dh,
    type = "n", ylim = c(min(dh), max(hh)),
    ylab = expression(mu), xlab = "",
    main = "", las = 1
  )
  mtext(bquote(1 - widehat(alpha) == .(a_pst_pokryti)), side = 1, line = 2.2)
  mtext(paste("n = ", n), side = 1, line = 3.4)
  segments(x, dh, x, hh, col = "grey40")
  segments(x[rozhodnuti == 0], dh[rozhodnuti == 0], x[rozhodnuti == 0], 
           hh[rozhodnuti == 0], col = "red")
  abline(h = mu)

  SE <- sqrt(a_alpha * (1 - a_alpha) / M)
  dh_alpha <- 1 - a_alpha - qnorm(1 - alpha / 2) * SE #
  hh_alpha <- 1 - a_alpha - qnorm(alpha / 2) * SE #
  tab <- data.frame(spolehlivost = 1 - alpha, a_pst_pokryti, dh_alpha, hh_alpha)
  return(tab)
}

HistWald <- function(mu0, n, M = 500,
                   mu1 = 20, mu2 = mu1, 
                   sigma1 = 10, sigma2 = sigma1, 
                   p = 0.9, alpha = 0.05,
                   main) {
  ## vypocet Z_W statistiky Waldova testu o stredni hodnote pri znamem rozptylu 
  ## a vykresleni histogramu obecne pro smes norm. rozdeleni
  ## mu0 ... hodnota mu z H0
  ## n ... rozsah nah. vyberu
  ## M ... pocet nah. vyberu
  ## mu1 ... stredni hodnota prvniho rozdeleni
  ## mu2 ... stredni hodnota druheho rozdeleni
  ## sigma1 ... sd prvniho rozdeleni
  ## sigma2 ... sd druheho rozdeleni
  ## p ... parametr smesi rozdeleni
  ## alpha ... hladina vyznamnosti
  ## main ... titulek histogramu
  
  X <- matrix(NA, M, n)
  for (i in 1:M) {
    bin <- rbinom(n, 1, p)
    X[i, ][bin == 1] <- rnorm(sum(bin), mu1, sigma1)
    X[i, ][bin == 0] <- rnorm(n - sum(bin), mu2, sigma2)
  }

  m <- apply(X, 1, mean)
  zW <- (m - mu0) / (sigma1 / sqrt(n))
  d <- hist(zW, plot = F)$dens
  xfit <- seq(min(zW) - 10, max(zW) + 10, length = 512)
  yfit <- dnorm(xfit)
  alpha_aktual <- sum(abs(zW) > qnorm(1 - alpha / 2)) / M
  #akt.p.pokryti <- 1 - alpha_aktual

  hist(zW,
    prob = T,  xlim = c(-4, 4), ylim = c(0, max(yfit, d)),
    density = 40, col = "dodgerblue", xlab = "",
    ylab = "relativni cetnost", main = "", las = 1
  )
  box(bty = "o")
  mtext(expression(z[W]), side = 1, line = 2.2)
  mtext(bquote(paste(alpha == .(alpha), "; ", widehat(alpha) == .(alpha_aktual))),
    side = 1,
    line = 3.3
  )
  mtext(main, side = 1, line = 4.5)
  lines(xfit, yfit, lwd = 2, col = "dodgerblue4")
}

RozdeleniNecentr <- function(mu, mu0 = 150, sigma = 10, sigma2 = sigma,
                      M = 2000, n = 50, p = 0.9,
                      main = "", pozice = "topleft") {
  ## rozdeleni test. statistiky pro test o mu pri znamem sigma
  ## mu ... str. hodnota rozdeleni
  ## mu0 ... hodnota mu z H0
  ## sigma ... sd rozdeleni
  ## sigma2 ... sd druheho rozdeleni ve smesi
  ## M ... pocet nah. vyberu
  ## n ... rozsah nah. vyberu
  ## p ... parametr smesi rozdeleni
  ## main ... popis
  ## pozice ... pozice legendy
  
  X <- matrix(NA, M, n)
  for (i in 1:M) {
    bin <- rbinom(n, 1, p)
    X[i, ][bin == 1] <- rnorm(sum(bin == 1), mu, sigma)
    X[i, ][bin == 0] <- rnorm(sum(bin == 0), mu, sigma2)
  }
  m <- apply(X, 1, mean)
  zW <- (m - mu0) / sigma * sqrt(n)
  lambda <- (mu - mu0) / sigma * sqrt(n)

  xfit <- seq(from = min(zW) - 5, to = max(zW) + 5, length = 512)
  yfit <- dnorm(xfit)
  zfit <- dnorm(xfit, mean = lambda)

  hist(zW,
    prob = T, ylab = "relativni cetnost", breaks = 20, xlab = "", col = "dodgerblue",
    xlim = c(min(zW) - 5, max(zW) + 5), density = 40, ylim = c(0, max(yfit) + 0.05), main = "", las = 1
  )
  box(bty = "o")
  mtext(expression(z[W]), side = 1, cex = 1, line = 2.2)
  mtext(bquote(paste(n == .(n), ", ", mu[0] == .(mu0))),
    side = 1, cex = 1, line = 3.3
  )
  mtext(main, side = 1, line = 4.5)
  lines(xfit, yfit, col = "red", lwd = 2)
  lines(xfit, zfit, col = "dodgerblue4", lwd = 2)

  legend(pozice,
    fill = c("dodgerblue2", "red", "dodgerblue4"), bty = "n",
    cex = 0.9, legend = c(expression(z[W]), "centralni rozd.", "necentralni rozd.")
  )
}

SilaExakt <- function(mu0 = 0, mu, sigma = 1,
                      n, alpha = 0.05, 
                      alternative = "two.sided") {
  ## silofunkce 
  ## mu0 ... hodnota mu z H0
  ## mu ... stredni hodnota rozdeleni
  ## sigma ... sd
  ## n ... rozsah nah. vyberu
  ## alpha ... hladina vyznamnosti
  ## alternative ... typ alternativy
  
  if (alternative == "two.sided") { # oboustranna alternativa
    sila <- pnorm(qnorm(alpha / 2) - (mu - mu0) / sigma * sqrt(n)) +
      pnorm(qnorm(alpha / 2) + (mu - mu0) / sigma * sqrt(n))
  }

  if (alternative == "greater") { # pravostranna alternativa
    sila <- pnorm(qnorm(alpha) + (mu - mu0) / sigma * sqrt(n))
  }

  if (alternative == "less") { # levostranna alternativa
    sila <- 1 - pnorm(qnorm(1 - alpha) + (mu - mu0) / sigma * sqrt(n))
  }
  return(sila)
}

PlotSila <- function(mu0 = 150, mu, sigma = 10, 
                     n, alpha = 0.05,
                     alternative = "two.sided") {
  barva <- c("dodgerblue4", "dodgerblue3", "dodgerblue1", "darkturquoise")
  plot(mu, SilaExakt(mu = mu, n = n[1]),
    type = "n",
    ylim = c(0, 1), xlab = expression(mu),
    ylab = expression(paste("silofunkce  ", 1 - beta)),
    las = 1
  )

  for (i in 1:length(n)) {
    sila <- SilaExakt(
      mu0 = mu0, mu, sigma = sigma, n = n[i],
      alpha = 0.05, alternative = alternative
    )
    lines(mu, sila, col = barva[i])
  }
  legend("right",
    col = barva, bty = "n", lty = c(1, 1),
    legend = paste("n =", n), lwd = 2
  )
  abline(h = alpha, col = "grey60", lty = 2)
}

SilaAprox <- function(mu0, mu, sigma, n, alpha) {
  ## vypocet aproximaticke silofunkce
  ## mu0 ... hodnota mu0 z H0
  ## mu ... stred. hodnota
  ## sigma ... sm. odchylka
  ## n ... rozsah vyberu
  ## alpha ... hladina vyznamnosti
  
  sila <- pnorm(qnorm(alpha / 2) + abs(mu0 - mu) / sigma * sqrt(n))
}

MinZTest <- function(mu, mu0 = 150, sigma = 10,
                      alpha = 0.05, sila = 0.8,
                      alternative = "two.sided") {
  ## vypocet minimalniho rozsahu pro Z-test
  ## mu ... str. hodnota
  ## mu0 ... hodnota mu z H0
  ## alpha ... hladina vyznamnosti
  ## sila ... sila testu
  ## alternative ... alternativa
  
  if (alternative == "two.sided") {
    n <- (qnorm(sila) - qnorm(alpha / 2))^2 / abs(mu - mu0)^2 * sigma^2
  }

  if (alternative == "greater") {
    n <- (qnorm(sila) - qnorm(alpha))^2 / (mu - mu0)^2 * sigma^2
  }

  if (alternative == "less") {
    n <- (qnorm(sila) - qnorm(alpha))^2 / (mu0 - mu)^2 * sigma^2
  }
  return(ceiling(n))
}

SilaAnimace <- function(mu0, mu, sigma, n,
                        alpha = 0.05, alternative = 'two.sided') {
  ## funkce pro animaci posunu necentralniho rozdeleni a silofunkce
  ## mu0 ... hodnota mu z H0
  ## mu ... stredni hodnota rozdeleni
  ## sigma ... sm. odchylka
  ## n ... rozsah nah. vyberu
  ## alpha ... hladina vyznamnosti
  ## alternative ... alternativa
  
  lambda <- (mu - mu0) / sigma * sqrt(n)
  x <- seq(-10, 10, length = 512)
  y <- dnorm(x)
  l <- dnorm(x, mean = lambda)
  
  par(mfrow = c(1, 2), mar = c(5, 4, 1, 1))
  plot(x, y, type = 'n', ylim = c(-0.05, 0.4), 
       xlab = '', ylab = 'f(x)', las = 1)
  mtext(expression(z[W]), side = 1, line = 2.2)
  mtext(bquote(paste(n == .(n), ', ', mu[0] == .(mu0), ', ',
                     mu == .(mu))), 
        side = 1, cex = 1, line = 3.3)
  
  q1 <- qnorm(alpha / 2)
  q2 <- qnorm(1 - alpha / 2)
  
  polygon(c(min(x), max(x[x <= q1]), max(x[x <= q1]), rev(x[x <= q1])), 
          c(0, 0, y[x == max(x[x <= q1])], rev(y[x %in% x[x <= q1]])), 
          col = 'red', density = 40)
  
  polygon(c(min(x[x >= q2]), max(x), rev(x[x >= q2]), min(x[x >= q2])),
          c(0, 0, rev(y[x %in% x[x >= q2]]), y[x == min(x[x >= q2])]),
          col = 'red', density = 40)
  
  lines(x, y, col = 'red')
  
  
  polygon(c(min(x), max(x[x <= q1]), max(x[x <= q1]), rev(x[x <= q1])), 
          c(0, 0, l[x == max(x[x <= q1])], rev(l[x %in% x[x <= q1]])),
          col = 'dodgerblue', density = 30)
  
  polygon(c(min(x[x >= q2]), max(x), rev(x[x >= q2]), min(x[x >= q2])), 
          c(0, 0, rev(l[x %in% x[x >= q2]]), l[x == min(x[x >= q2])]),
          col = 'dodgerblue', density = 30)
  
  lines(x, l, col = 'dodgerblue')
  legend('bottomleft', lwd = 2, col = c('red', 'dodgerblue'), bty = 'n', horiz = T,
         legend = c('centralni rozd.', 'necentralni rozd.'))
  
  mu1  <- seq(mu0 - 20, mu0 + 20, length = 512)
  
  sila_ex <- SilaExakt(mu0 = mu0, mu = mu1, sigma = sigma,
                       n = n, alternative = 'two.sided')
  plot(mu1, sila_ex, col = 'dodgerblue', type = 'l', xlab = ' ', 
       ylab = 'silofunkce', las = 1, ylim = c(-0.1, 1))
  mtext(expression(mu), side = 1, line = 2.2)
  
  sila_akt <- SilaExakt(mu0 = mu0, mu = mu, sigma = sigma,
                        n = n, alternative = 'two.sided')
  points(mu, sila_akt, pch = 19, col = 'dodgerblue3', cex = 1.2)
  legend('bottomleft', col = c('dodgerblue3', 'dodgerblue4'), lwd = c(2, NA), pch = c(NA, 19),
         legend = c('krivka ex. sily', 'okamzita ex. sila'), bty = 'n', horiz = T)
  abline(h = alpha, lty = 2, col = 'grey50')
}



UW <- function(tw, n){
  ## funkce pro vypocet test. stat. Uw
  ## tw ... Waldova t-test. stat. tw
  ## n ... rozsah nah. vyberu
  uw <- tw ^ 2 * (n / (n - 1))
  return(uw)
}

US <- function(tw, n){
  ## funkce pro vypocet test. stat. US
  ## tw ... Waldova t-test. stat. tw
  ## n ... rozsah nah. vyberu
  us <- n * tw ^ 2 / (n - 1 + tw ^ 2)
  return(us)
}

ULR <- function(tw, n){
  ## funkce pro vypocet test. stat. ULR
  ## tw ... Waldova t-test. stat. tw
  ## n ... rozsah nah. vyberu
  ulr <- n * log(1 + tw ^ 2 / (n - 1))
  return(ulr)
}

TStat <- function(mu, sigma, M = 1000, n, type = 'tW') {
  ## mu ... stredni hodnota
  ## sigma ... sd
  ## M ... pocet nah. vyberu
  ## n ... rozsah nah. vyberu
  ## type ... typ test. statistiky
  
  x <- rnorm(M * n, mu, sigma)
  X <- matrix(x, nrow = M)
  
  m <- apply(X, 1, mean)
  s <- apply(X, 1, sd)
  tW <- (m - mu) / (s / sqrt(n))
  
  if (type == 'tW') { 
    X <- tW
    main <- expression(t[W]) 
    xlim = c(-7, 7) 
    xx <- seq(-7, 7, length = 512)
    yy <- dt(xx, n - 1)}
  
  if (type == 'tW2') { 
    X <- tW ^ 2
    main <- expression(t[W]^2)
    xlim = c(0, 15)
    xx <- seq(0, 15, length = 512)
    yy <- df(xx, 1, n - 1)}
  
  if (type == 'uW') { 
    X <- UW(tW, n) 
    main <- expression(U[W])
    xlim = c(0, 10)
    xx <- seq(0, 15, length = 512)
    yy <- dchisq(xx, 1)}
  
  if (type == 'uS') { 
    X <- US(tW, n)
    main <- expression(U[S])
    xlim = c(0, 10)
    xx <- seq(0, 15, length = 512)
    yy <- dchisq(xx, 1)}
  
  if (type == 'uLR') { 
    X <- ULR(tW, n)
    main <- expression(U[LR])
    xlim = c(0, 10)
    xx <- seq(0, 15, length = 512)
    yy <- dchisq(xx, 1)}
  
  hist(X, prob = T, density = 40, col = 'coral', main = '', 
       ylim = c(0, 0.5), xlim = xlim,
       xlab = '', las = 1, ylab = 'relativni cetnost',
       breaks = 15)
  box(bty = 'o')
  lines(xx, yy, col = 'coral4', lwd = 2)
  mtext(main, side = 1, line = 2.2)
  mtext(bquote(paste(n == .(n))), side = 1, line = 3.3)
}
  
PorovnaniU <- function(n, M = 1000, mu0, mu, sigma, plot = T) {
  
  X <- t(replicate(M, rnorm(n, mu, sigma)))
  m <- apply(X, 1, mean)
  s <- apply(X, 1, sd)
  tW <- (m - mu0) / s * sqrt(n)
  uW <- UW(tW, n)
  uS <- US(tW, n)
  uLR <- ULR(tW, n)

  prumery <- c(mean(uW), mean(uS), mean(uLR))
  if (plot == T) {
    plot(uW,
      type = "n", ylim = c(0, 1), xlim = c(0, 4),
      xlab = "", ylab = "hustota",
      main = "", las = 1
    )
    mtext("x", side = 1, line = 2)
    mtext(paste("n =", n), side = 1, line = 3)
    lines(density(uW, from = 0), col = "red")
    lines(density(uS, from = 0), col = "blue")
    lines(density(uLR, from = 0), col = "darkgreen")
    legend("topright",
      col = c("red", "darkgreen", "blue"),
      legend = c(expression(u[W]), expression(u[LR]), expression(u[S])),
      lwd = 2, bty = "n"
    )
    abline(v = 0, lty = 2, col = "gray60")
  }
  return(prumery = prumery)
}